@@ -42,15 +42,9 @@ pub(crate) fn pick_translation(subgizmo: &SubGizmo, ui: &Ui, ray: Ray) -> Option
 }
 
 pub(crate) fn draw_translation(subgizmo: &SubGizmo, ui: &Ui) {
-    let transform = if subgizmo.config.local_space() {
-        Mat4::from_rotation_translation(subgizmo.config.rotation, subgizmo.config.translation)
-    } else {
-        Mat4::from_translation(subgizmo.config.translation)
-    };
-
     let painter = Painter3d::new(
         ui.painter().clone(),
-        subgizmo.config.view_projection * transform,
+        subgizmo.config.view_projection * translation_transform(subgizmo),
         subgizmo.config.viewport,
     );
 
@@ -60,25 +54,17 @@ pub(crate) fn draw_translation(subgizmo: &SubGizmo, ui: &Ui) {
 
     let width = subgizmo.config.scale_factor * subgizmo.config.visuals.stroke_width;
     let length = subgizmo.config.scale_factor * subgizmo.config.visuals.gizmo_size;
-    let arrow_half_width = width * 2.0;
-    let arrow_length = arrow_half_width + length * 0.1;
+    let arrow_length = width * 2.4;
     let length = length - arrow_length;
 
     let start = direction * width;
     let end = direction * length;
 
     painter.line_segment(start, end, (subgizmo.config.visuals.stroke_width, color));
-
-    let cross = if subgizmo.config.local_space() {
-        direction.cross(subgizmo.config.rotation.inverse() * subgizmo.config.view_forward())
-    } else {
-        direction.cross(subgizmo.config.view_forward())
-    } * arrow_half_width;
-
-    painter.polygon(
-        &[end + cross, end - cross, end + (direction * arrow_length)],
-        color,
-        (0.0, color),
+    painter.arrow(
+        end,
+        end + direction * arrow_length,
+        (subgizmo.config.visuals.stroke_width * 1.2, color),
     );
 }
 
@@ -95,18 +81,18 @@ pub(crate) fn update_translation(subgizmo: &SubGizmo, ui: &Ui, ray: Ray) -> Opti
         new_point = state.start_point + new_delta;
     }
 
-    let (scale, rotation, mut translation) =
-        subgizmo.config.model_matrix.to_scale_rotation_translation();
-    translation += new_point - state.last_point;
-
     subgizmo.update_state_with(ui, |state: &mut TranslationState| {
         state.last_point = new_point;
         state.current_delta = new_delta;
     });
 
     Some(GizmoResult {
-        transform: Mat4::from_scale_rotation_translation(scale, rotation, translation)
-            .to_cols_array_2d(),
+        transform: Mat4::from_scale_rotation_translation(
+            subgizmo.config.scale,
+            subgizmo.config.rotation,
+            subgizmo.config.translation + new_point - state.last_point,
+        )
+        .to_cols_array_2d(),
         mode: GizmoMode::Translate,
         value: state.current_delta.to_array(),
     })
@@ -193,18 +179,18 @@ pub(crate) fn update_translation_plane(
         new_point = state.start_point + new_delta;
     }
 
-    let (scale, rotation, mut translation) =
-        subgizmo.config.model_matrix.to_scale_rotation_translation();
-    translation += new_point - state.last_point;
-
     subgizmo.update_state_with(ui, |state: &mut TranslationState| {
         state.last_point = new_point;
         state.current_delta = new_delta;
     });
 
     Some(GizmoResult {
-        transform: Mat4::from_scale_rotation_translation(scale, rotation, translation)
-            .to_cols_array_2d(),
+        transform: Mat4::from_scale_rotation_translation(
+            subgizmo.config.scale,
+            subgizmo.config.rotation,
+            subgizmo.config.translation + new_point - state.last_point,
+        )
+        .to_cols_array_2d(),
         mode: GizmoMode::Translate,
         value: state.current_delta.to_array(),
     })
@@ -248,7 +234,7 @@ fn translation_transform(subgizmo: &SubGizmo) -> Mat4 {
     }
 }
 
-fn translation_plane_binormal(direction: GizmoDirection) -> Vec3 {
+pub(crate) fn translation_plane_binormal(direction: GizmoDirection) -> Vec3 {
     match direction {
         GizmoDirection::X => Vec3::Y,
         GizmoDirection::Y => Vec3::Z,
@@ -257,7 +243,7 @@ fn translation_plane_binormal(direction: GizmoDirection) -> Vec3 {
     }
 }
 
-fn translation_plane_tangent(direction: GizmoDirection) -> Vec3 {
+pub(crate) fn translation_plane_tangent(direction: GizmoDirection) -> Vec3 {
     match direction {
         GizmoDirection::X => Vec3::Z,
         GizmoDirection::Y => Vec3::X,
@@ -266,12 +252,12 @@ fn translation_plane_tangent(direction: GizmoDirection) -> Vec3 {
     }
 }
 
-fn translation_plane_size(subgizmo: &SubGizmo) -> f32 {
+pub(crate) fn translation_plane_size(subgizmo: &SubGizmo) -> f32 {
     subgizmo.config.scale_factor
         * (subgizmo.config.visuals.gizmo_size * 0.1 + subgizmo.config.visuals.stroke_width * 2.0)
 }
 
-fn translation_plane_local_origin(subgizmo: &SubGizmo) -> Vec3 {
+pub(crate) fn translation_plane_local_origin(subgizmo: &SubGizmo) -> Vec3 {
     let offset = subgizmo.config.scale_factor * subgizmo.config.visuals.gizmo_size * 0.4;
 
     let a = translation_plane_binormal(subgizmo.direction);
@@ -279,7 +265,7 @@ fn translation_plane_local_origin(subgizmo: &SubGizmo) -> Vec3 {
     (a + b) * offset
 }
 
-fn translation_plane_global_origin(subgizmo: &SubGizmo) -> Vec3 {
+pub(crate) fn translation_plane_global_origin(subgizmo: &SubGizmo) -> Vec3 {
     let mut origin = translation_plane_local_origin(subgizmo);
     if subgizmo.config.local_space() {
         origin = subgizmo.config.rotation * origin;
